module ConfigFile(
    loadStdConfiguration,
    configLookupBool,
    configLookup,
    configLookupList
    ) where

import Char
import System
import System.IO.Unsafe

import Signal

type ConfigFile = [(String, String)]


getConfiguration :: String -> IO ConfigFile
getConfiguration s = do
    e <- getEnv "HOME"
    c <- readFile (e ++ "/" ++ s)
    return $ parseConfigFile c
    
dumpConfigFile :: ConfigFile -> String
dumpConfigFile cf = unlines $ map dc cf where
    dc (n,v) = n ++ " " ++ v

mergeConfigFile :: ConfigFile -> ConfigFile -> ConfigFile
mergeConfigFile c1 c2 = c1 ++ c2

isPropName c = isAlphaNum c || c `elem` "-_"

parseConfigFile :: String -> ConfigFile
parseConfigFile s = concatMap bl (fixup $ lines (uncomment s)) where
    uncomment ('#':xs) = uncomment (dropWhile (/= '\n') xs)
    uncomment ('-':'-':xs) = uncomment (dropWhile (/= '\n') xs)
    uncomment (x:xs) = x:uncomment xs
    uncomment [] = []
    fixup (x:y@(c:_):xs) | isSpace c = fixup ((x ++ y):xs)
    fixup (x:xs) = x: fixup xs
    fixup [] = []
    bl s = let (n,r) = span isPropName (dropWhile isSpace s) in
	if null n then [] else [(n,dropWhile isSpace r)]


{-# NOINLINE config_file_var #-}
config_file_var :: SVar ConfigFile
config_file_var = unsafePerformIO $ newSVar []


loadStdConfiguration :: String -> IO ()
loadStdConfiguration f = do
    c <- getConfiguration f
    writeSVar config_file_var c
    return ()

loadExactConfiguration :: String -> IO ()
loadExactConfiguration f = do
    s <- readFile f 
    let c = parseConfigFile s
    writeSVar config_file_var c
    return ()


configLookupBool k = do
    vs <- readSVar config_file_var
    case lookup k vs of 
	Just s | cond -> return True where
	    cond = s' == "true" || s' == "yes" || s' == "on" || s' == "y" || s' == "t"
	    s' = map toLower s
	_ -> return False
    

configLookup k = do
    vs <- readSVar config_file_var
    return $ lookup k vs


configLookupList k = do
    vs <- readSVar config_file_var
    return $ [ y| (x,y) <- vs, x == k]

